/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.collection;


/**
 * Simple implementation of extensible byte array.
 * <p>
 * <b>Purpose</b><br>
 *
 * <p>
 * <b>Implementation</b><br>
 * <p>
 * <b>Examples</b><br>
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/toolkit/src/nl/justobjects/toolkit/collection/ByteBuffer.java,v $<p>
 *
 * @version $Id: ByteBuffer.java,v 1.3 2003/01/06 00:23:48 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 */
public class ByteBuffer {

	/**
	 * The buffer where data is stored.
	 */
	protected byte buf[];

	/**
	 * The number of valid bytes in the buffer.
	 */
	protected int count;

	/**
	 * Creates a new byte buffer. The buffer capacity is
	 * initially 32 bytes, though its size increases if necessary.
	 */
	public ByteBuffer() {
		this(32);
	}

	/**
	 * Creates a new byte buffer, with a buffer capacity of
	 * the specified size, in bytes.
	 *
	 * @param   size   the initial size.
	 * @exception  IllegalArgumentException if size is negative.
	 */
	public ByteBuffer(int size) {
		if (size < 0) {
			throw new IllegalArgumentException("Negative initial size: "
					+ size);
		}
		buf = new byte[size];
	}

	/**
	 * Add the specified byte to this byte buffer.
	 *
	 * @param   b   the byte to be written.
	 */
	public void add(byte b) {
		int newcount = count + 1;
		if (newcount > buf.length) {
			byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
			System.arraycopy(buf, 0, newbuf, 0, count);
			buf = newbuf;
		}
		buf[count] = (byte) b;
		count = newcount;
	}

	/**
	 * Adds bytes from the specified byte array
	 *
	 * @param   b     the data.
	 */
	public void add(byte b[]) {
		add(b, 0, b.length);
	}

	/**
	 * Adds <code>len</code> bytes from the specified byte array
	 * starting at offset <code>off</code> to this byte buffer.
	 *
	 * @param   b     the data.
	 * @param   off   the start offset in the data.
	 * @param   len   the number of bytes to write.
	 */
	public void add(byte b[], int off, int len) {
		if ((off < 0) || (off > b.length) || (len < 0) ||
				((off + len) > b.length) || ((off + len) < 0)) {
			throw new IndexOutOfBoundsException();
		} else if (len == 0) {
			return;
		}

		int newcount = count + len;
		if (newcount > buf.length) {
			byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
			System.arraycopy(buf, 0, newbuf, 0, count);
			buf = newbuf;
		}
		System.arraycopy(b, off, buf, count, len);
		count = newcount;
	}

	public void add(char c) {
		add((byte) c);
	}

	public void add(String s) {
		add(s.getBytes(), 0, s.length());
	}

	/**
	 * Resets the <code>count</code> field of this byte buffer
	 * to zero
	 *
	 */
	public void reset() {
		count = 0;
	}

	/**
	 * Creates a newly allocated byte array. Its size is the current
	 * size of this buffer and the valid contents of the buffer
	 * have been copied into it.
	 *
	 * @return  the current contents of this buffer, as a byte array.
	 * @see     java.io.ByteArrayOutputStream#size()
	 */
	public byte[] toByteArray() {
		byte newbuf[] = new byte[count];
		System.arraycopy(buf, 0, newbuf, 0, count);
		return newbuf;
	}

	/**
	 * Returns the current size of the buffer.
	 *
	 * @return  the value of the <code>count</code> field, which is the number
	 *          of valid bytes in this buffer.
	 * @see     java.io.ByteArrayOutputStream#count
	 */
	public int size() {
		return count;
	}

	/**
	 * Converts the buffer's contents into a string, translating bytes into
	 * characters according to the platform's default character encoding.
	 *
	 * @return String translated from the buffer's contents.
	 * @since   JDK1.1
	 */
	public String toString() {
		return new String(buf, 0, count);
	}


}

/*
 * $Log: ByteBuffer.java,v $
 * Revision 1.3  2003/01/06 00:23:48  just
 * moved devenv to linux
 *
 * Revision 1.2  2002/11/14 20:25:19  just
 * reformat of code only
 *
 * Revision 1.1  2001/08/16 11:28:57  kstroke
 * new
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/