package nl.justobjects.toolkit.io;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;
import java.util.Vector;


/** Various I/O and Unix-type utilities.
 *
 * @version $Id: IO.java,v 1.27 2003/01/06 00:23:48 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
public class IO {
	public static int bufferSize = 4096;

	/** Recursively copies directory. */
	public static void cpdir(String from, String to) throws IOException {
		//p("enter cpdir "+from+" to "+to);

		// If not existent creates destination dir.
		mkdir(to);

		// Copy all subdirs and files in dir.
		File fromDir = new File(from);
		File toDir = new File(to);
		if (fromDir.getAbsolutePath().equals(toDir.getAbsolutePath())) {
			// from and to are equal: nothing to do
			return;
		}
		String[] fileList = fromDir.list();

		for (int i = 0; i < fileList.length; i++) {
			String fromPath = from + File.separator + fileList[i];

			String toPath = to + File.separator + fileList[i];

			// If subdirectory call ourselves to copy it
			if (new File(fromPath).isDirectory()) {
				//p("copying dir "+fromPath+" to "+toPath);
				cpdir(fromPath, toPath);
			} else {
				// Copies file to destination dir.
				cp(fromPath, toPath);
			}
		}
	}

	/** Standard UNIX-style file copy. */
	public static void cp(String from, String to) throws IOException {
		File fromFile = new File(from);
		File toFile = new File(to);

		// UNIX-style: if destination is dir create file ith same name there
		if (toFile.isDirectory()) {
			to = to + File.separator + fromFile.getName();
			toFile = new File(to);
		}

		// p("copying file "+from+" to "+to);
		copy(new FileInputStream(fromFile), new FileOutputStream(toFile));
	}

	/** Create directory path if none exists for a filename. */
	public static void createFilePath(String fileName) throws IOException {
		p("createFilePath: " + fileName);
		File file = new File(fileName);
		if (file.getParent() != null) {
			if (!(new File(file.getParent()).exists())) {
				new File(file.getParent()).mkdirs();
			}
		}
	}

	/** Execute external program. */
	public static String exec(String[] commandLine) throws IOException {
		// Execute
		Process child = Runtime.getRuntime().exec(commandLine);

		// Get the child's output (=our input)
		InputStream is = child.getInputStream();

		// Capture output
		int c;
		StringBuffer sb = new StringBuffer();
		while ((c = is.read()) != -1) {
			sb.append((char) c);
		}

		is.close();

		// Wait for child process to finish
		try {
			child.waitFor();
		} catch (InterruptedException ie) {
		}

		// Did we fail ?
		if (child.exitValue() != 0) {
			throw new IOException("exit value =" + child.exitValue());
		}

		// Retrun process output.
		return sb.toString();
	}

	/** Very basic grep: return lines in file that contain exact string. */
	public static String[] grep(String aFile, String aGrepString) throws IOException {
		Vector vector = new Vector(3);
		BufferedReader br = new BufferedReader(new FileReader(aFile));
		String nextLine = null;
		try {
			nextLine = br.readLine();
			while (nextLine != null) {
				if (nextLine.indexOf(aGrepString) != -1) {
					vector.addElement(nextLine);
				}
				nextLine = br.readLine();
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (br != null) {
				try {
					br.close();
				} catch (IOException ignore) {
				}
			}
		}

		String[] result = new String[vector.size()];
		vector.copyInto(result);
		return result;
	}

	/** Standard UNIX-style file move. */
	public static void mv(String from, String to) throws IOException {
		cp(from, to);
		rm(from);
	}

	/** Creates multiple directories according to path. */
	public static void mkdir(String dirPath) throws IOException {
		new File(dirPath).mkdirs();
	}

	/** Compare modification dates of two files. */
	public static boolean newer(String file1, String file2) {
		long file1Time = new File(file1).lastModified();
		long file2Time = new File(file2).lastModified();
		return (file1Time > file2Time);
	}

	/** Compare modification dates of two files. */
	public static boolean newer(File file1, String file2) {
		long file1Time = file1.lastModified();
		long file2Time = new File(file2).lastModified();
		return (file1Time > file2Time);
	}

	/** Replace < in String with &lt; (HTML/XML escape) */
	public static String escapeMarkup(String string) {
		if (string.indexOf("<") == -1) {
			return string;
		}
		StringBuffer stringBuffer = new StringBuffer(string);
		int ix = -1;
		while ((ix = string.indexOf("<")) > -1) {
			stringBuffer.setCharAt(ix, '&');
			stringBuffer.insert(ix + 1, "lt;");
			string = stringBuffer.toString();
			// p("string="+string);
		}
		return string;
	}

	/** Make recursive list of directories and files. */
	public static String[] dir2StringArray(String dir, String skip) {
		File f = new File(dir);
		String[] theList = null;
		if (f.isDirectory()) {
			theList = f.list();
			for (int i = 0; i < theList.length; i++) {
				File tmpFile = new File(theList[i]);
				theList[i] = dir + "/" + tmpFile.getPath();
				if (tmpFile.isDirectory()) {
					String[] addList = dir2StringArray(theList[i], skip);
					String[] newList = new String[theList.length + addList.length];
					System.arraycopy(theList, 0, newList, 0, theList.length);
					System.arraycopy(addList, 0, newList, theList.length, addList.length);
					theList = newList;
				}
			}
		}
		return theList;
	}

	/** Return contents of file as array of String.
	 *  @param name the filename
	 */
	public static String[] FileToStringArray(String fileName) {
		File f = new File(fileName);
		FileInputStream fis = null;
		Vector vector = new Vector(20);
		DataInputStream dis = null;

		try {
			fis = new FileInputStream(f);
			dis = new DataInputStream(fis);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return DataInputStreamToStringArray(dis);
	}

	/** Return contents of DataInputStream as String array.
	 *  @param dis the DataInputStream
	 */
	public static String[] DataInputStreamToStringArray(DataInputStream dis) {
		Vector vector = new Vector(20);
		try {
			String line = null;
			line = dis.readLine();
			while (line != null) {
				vector.addElement(line + "\n");
				line = dis.readLine();
			}
			dis.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		String[] result = new String[vector.size()];
		vector.copyInto(result);
		return result;
	}

	/** Return contents of file as String.
	 *  @param name the filename
	 */
	public static String FileToString(String fileName) {
		FileInputStream fis = null;
		StringBuffer result = new StringBuffer();
		result.append("");
		try {
			fis = new FileInputStream(fileName);
			DataInputStream dis = new DataInputStream(fis);
			String line = null;
			line = dis.readLine();
			while (line != null) {
				result.append(line + "\n");
				line = dis.readLine();
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (Exception ignore) {
				}
			}
		}
		return result.toString();
	}

	/** Copy resource found in CLASSPATH to a File.
	 *  @param resourceName the resource name
	 *  @param fileName the file to copy to
	 */
	public static void resourceToFile(String resourceName, String fileName) throws Exception {
		InputStream is = null;
		File file = new File(fileName);
		FileOutputStream os = null;
		try {
			// is = ClassLoader.getSystemResourceAsStream(resourceName);
			is = IO.class.getClassLoader().getResourceAsStream(resourceName);
			os = new FileOutputStream(file);
			copy(is, os);
		} catch (IOException e) {
			throw e;
		} finally {
			os.close();
			is.close();
		}
	}

	/**  Write a String to a File
	 *  @param name the filename
	 *  @param toBeWritten the String to write
	 */
	public static void StringToFile(String name, String toBeWritten) {
		StringToFile(name, toBeWritten, false);
	}

	/**  Write a String to a File
	 *  @param name the filename
	 *  @param toBeWritten the String to write
	 */
	public static void StringToFile(String name, String toBeWritten, boolean append) {
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(name, append);
			StringToOutputStream(fos, toBeWritten);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/** Write a String array to a File
	 *  @param name the filename
	 *  @param toBeWritten the String array to write
	 */
	public static void StringArrayToFile(String name, String[] toBeWritten) {
		File f = new File(name);
		FileOutputStream fos = null;
		OutputStreamWriter osw = null;

		try {
			fos = new FileOutputStream(f);
			osw = new OutputStreamWriter(fos);
			for (int i = 0; i < toBeWritten.length; i++) {
				osw.write(toBeWritten[i], 0, toBeWritten[i].length());
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				osw.close();
			} catch (IOException ignore) {
			}
		}
	}

	/** Write a String to an OutputStream
	 *  @param os the OutputStream
	 *  @param toBeWritten the String to write
	 */
	public static void StringToOutputStream(OutputStream os, String toBeWritten) {
		OutputStreamWriter osw = new OutputStreamWriter(os);
		try {
			osw.write(toBeWritten, 0, toBeWritten.length());
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				osw.close();
			} catch (IOException ignore) {
			}
		}
	}

	/** Copy Stream in to Stream out until EOF or exception. */
	public static void copy(InputStream in, OutputStream out) throws IOException {
		copy(in, out, -1);
	}

	/** Replace all tokens in a string with new token. */
	public static String replaceToken(String string, String fromToken, String toToken) {
		// Need to use our own StringBuffer version because replace() is not in JDK11 !!
		StringBuffer stringBuffer = new StringBuffer(string);

		int ix = -1;
		while ((ix = string.indexOf(fromToken)) > -1) {
			stringBuffer.replace(ix, ix + fromToken.length(), toToken);
			string = stringBuffer.toString();
			// System.out.println("replacing at="+ix);
		}
		return string;
	}

	/** Replace all tokens in a file with new token. */
	public static void replaceTokensInFile(String fileName, String fromToken, String toToken) {
		/* String string = IO.FileToString(fileName);
		String replaceString = replaceToken(string, fromToken, toToken);
    	IO.StringToFile(fileName, replaceString); */
		String[] fileArray = FileToStringArray(fileName);
		for (int i = 0; i < fileArray.length; i++) {
			if (fileArray[i].indexOf(fromToken) != -1) {
				fileArray[i] = replaceToken(fileArray[i], fromToken, toToken);
			}
		}
		StringArrayToFile(fileName, fileArray);
	}

	/** Copy Stream in to Stream out until EOF or exception. */
	public static void copy(InputStream in, OutputStream out, long byteCount) throws IOException {
		try {
			byte buffer[] = new byte[bufferSize];
			int len = bufferSize;

			if (byteCount >= 0) {
				while (byteCount > 0) {
					if (byteCount < bufferSize)
						len = in.read(buffer, 0, (int) byteCount);
					else
						len = in.read(buffer, 0, bufferSize);

					if (len == -1)
						break;

					byteCount -= len;
					out.write(buffer, 0, len);
				}
			} else {
				while (true) {
					len = in.read(buffer, 0, bufferSize);
					if (len == -1)
						break;
					out.write(buffer, 0, len);
				}
			}
		} finally {
			out.flush();
		}
	}


	public static void p(String s) {
		// System.out.println("IO: " + s);
	}

	/**
	 * Creates an Object from a bytearray containing a serialized Object.
	 * @param data[] the byte array containing the serialized object.
	 * @return Object the object.
	 */
	public static Object bytesToObject(byte data[]) {

		Object obj = null;
		Exception e = null;

		try {
			ByteArrayInputStream byteStream = new ByteArrayInputStream(data);
			ObjectInputStream inStream = new ObjectInputStream(byteStream);

			obj = inStream.readObject();
		} catch (StreamCorruptedException e1) {
			e = e1;
		} catch (OptionalDataException e2) {
			e = e2;
		} catch (InvalidClassException e3) {
			e = e3;
		} catch (IOException e4) {
			e = e4;
		} catch (ClassNotFoundException e5) {
			e = e5;
		}

		return obj;
	}

	/**
	 * Create an array of bytes from an Object.
	 * @param obj Het object.
	 * @return byte[] the serialized Object.
	 */
	public static byte[] objectToBytes(Object obj) {
		byte bytes[] = null;
		Exception e = null;

		try {
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			ObjectOutputStream outStream = new ObjectOutputStream(byteStream);

			outStream.writeObject(obj);
			bytes = byteStream.toByteArray();
		} catch (StreamCorruptedException e1) {
			e = e1;
		} catch (OptionalDataException e2) {
			e = e2;
		} catch (InvalidClassException e3) {
			e = e3;
		} catch (IOException e4) {
			e = e4;
		}
		return bytes;
	}

	/** Make Properties object from file or URL. */
	static public Properties getProperties(String name) {
		InputStream inputStream = null;

		try {
			if (name.startsWith("http://")) {
				inputStream = new URL(name).openStream();
			} else {
				inputStream = new FileInputStream(name);
			}
		} catch (FileNotFoundException e1) {
			return null;
		} catch (IOException e2) {
			return null;
		} catch (Exception e3) {
			return null;
		}

		return getProperties(inputStream);
	}

	/** Make Properties object from URL. */
	public static Properties getProperties(URL baseURL, String file) {
		URL url = null;
		try {
			url = new URL(baseURL, file);
		} catch (MalformedURLException e) {
			return null;
		}
		return getProperties(url);
	}

	/** Make Properties object from Properties InputStream. */
	public static Properties getProperties(InputStream inputStream) {
		Properties properties = new Properties();
		try {
			properties.load(inputStream);
		} catch (IOException e) {
			return null;
		}
		return properties;
	}

	/** Make Properties object from URL. */
	public static Properties getProperties(URL url) {
		InputStream inputStream;
		try {
			inputStream = url.openStream();
		} catch (IOException e) {
			return null;
		}
		return getProperties(inputStream);
	}

	/** Make Properties object from file retrieved through CLASSPATH. */
	public static Properties getPropertiesFromClassPath(String path) throws Exception {
		Properties properties = new Properties();

		// properties.load(ClassLoader.getSystemResourceAsStream(path));
		properties.load(IO.class.getClassLoader().getResourceAsStream(path));
		return properties;
	}

	/** Get real file name for file retrieved through CLASSPATH. */
	public static String getResourceFromClassPath(String path) throws Exception {
		URL url = IO.class.getClassLoader().getResource(path);
		// URL url = ClassLoader.getSystemResource(path);
		// System.out.println("file="+url.getFile());
		// NOTE: this does not work when the resource is in a .jar file!!!
		return url.getFile();
	}

	/** Standard UNIX-style file remove of file. */
	public static boolean rm(String fileName) throws IOException {
		return new File(fileName).delete();
	}

	/** Recursive remove of (non-empty) directory tree. */
	public static boolean rmdir(String dirName) throws IOException {
		File file = new File(dirName);

		String[] fileList = file.list();
		if (fileList != null) {
			for (int i = 0; i < fileList.length; i++) {
				String filePath = file.getPath() + File.separator + fileList[i];
				if (new File(filePath).isDirectory()) {
					if (!rmdir(filePath)) return false;
				} else {
					if (!rm(filePath)) return false;
				}
			}
		}
		return file.delete();
	}

	/** Return contents of URL as String.
	 *  @param name the filename
	 */
	public static String url2String(String anUrlString) {
		URL url = null;
		try {
			url = new URL(anUrlString);
		} catch (MalformedURLException e) {
			return null;
		}

		String result = "";
		try {
			InputStream is = url.openStream();
			DataInputStream dis = new DataInputStream(is);
			String line = null;
			line = dis.readLine();
			while (line != null) {
				result = result + line + "\n";
				line = dis.readLine();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return result;
	}

	/** Only used to test above methods. */
	public static void main(String[] args) {
		try {
			// String s = getResourceFromClassPath("nl/justobjects/toolkit/io/IO.class");
			// rmdir("e:/project/jo-keyworx/server/db/space/Space3");
			System.out.println(url2String("http://194.109.226.220:8005/test/stress-chat.xml"));
		} catch (Exception e) {
			p(e.toString());
		}
	}
}

/*
 * $Log: IO.java,v $
 * Revision 1.27  2003/01/06 00:23:48  just
 * moved devenv to linux
 *
 * Revision 1.26  2002/11/14 20:25:19  just
 * reformat of code only
 *
 * Revision 1.25  2002/07/04 15:34:07  just
 * removed println
 *
 * Revision 1.24  2002/06/18 10:28:30  just
 * added a grep method
 *
 * Revision 1.23  2002/05/23 15:06:07  just
 * file not closed in FileToString + now using StringBuffer
 *
 * Revision 1.22  2002/01/24 22:56:43  just
 * fixes for input buffer size (audio problem)
 *
 * Revision 1.21  2002/01/24 01:13:29  just
 * new utilities (createFilePath)
 *
 * Revision 1.20  2001/12/13 23:12:02  just
 * resource loading now uses IO.class.getClassLoader().getResource..()
 *
 * Revision 1.19  2001/12/11 14:04:38  just
 * changed StringBuffer13 back to StringBuffer (error in xmenu)
 *
 * Revision 1.18  2001/11/02 12:53:25  kstroke
 * added url2String
 *
 * Revision 1.17  2001/08/14 10:58:03  kstroke
 * fixes for javadoc comments
 *
 * Revision 1.16  2001/08/06 21:51:15  kstroke
 * now using StringBuffer13 iso StringBuffer (for JDK11 compile)
 *
 * Revision 1.15  2001/06/07 14:19:21  just
 * upd for rmdir
 *
 * Revision 1.14  2001/02/26 21:55:23  just
 * no changes
 *
 * Revision 1.13  2001/02/17 14:27:55  just
 * added comments and changed interface for expandIds()
 *
 * Revision 1.12  2001/02/11 20:25:51  just
 * added resourceToFile()
 *
 * Revision 1.11  2000/12/30 14:37:50  just
 * added exec()
 *
 * Revision 1.10  2000/12/29 16:26:18  just
 * added rm and mv commands
 *
 * Revision 1.9  2000/12/28 00:31:14  just
 * removed debug-println
 *
 * Revision 1.8  2000/12/28 00:20:02  just
 * optimized replaceTokensInFile()
 *
 * Revision 1.7  2000/12/09 16:36:20  just
 * added replaceTokensInFile()
 *
 * Revision 1.6  2000/10/12 09:47:36  just
 * text replacement util added
 *
 * Revision 1.5  2000/09/14 11:24:50  just
 * added URLtoString
 *
 * Revision 1.4  2000/08/10 19:27:29  just
 * changes for comments only
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
