/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.sys;

import java.util.HashSet;
import java.util.Set;

/**
 * Holds and parses a simple String[]-based commandline..
 * <p>
 * <b>Purpose</b><br>
 * All Java apps have a main(String[] args) entry method. In order
 * to make parsing easier the most common cases are implemented in
 * this class.
 * <p>
 * <b>Implementation</b><br>
 * Simple parsing stuff. No intelligent Unix-style getopt() yet.
 * <p>
 * <b>Examples</b><br>
 * All forms like -option or -option value or plain args can be used.
 * e.g. -verbose -o dir file
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/toolkit/src/nl/justobjects/toolkit/sys/CommandLine.java,v $<p>
 *
 * @version $Id: CommandLine.java,v 1.4 2003/01/06 00:23:49 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/01/06 00:23:49 $
 */

public class CommandLine {
	private String[] args;
	private Set usedParms = new HashSet();

	/** Single constructor with the args. */
	public CommandLine(String[] theArgs) {
		args = theArgs;
	}

	/** Get arg as integer e.g. -port 8080. */
	public int getIntArg(String name) {
		String arg = getStringArg(name);
		if (arg == null) {
			return -1;
		}

		return Integer.parseInt(arg);
	}

	/**
	 * If there exists an argument like
	 *     -name arg
	 * returns 'arg'.
	 * Else, returns null.
	 */
	public String getStringArg(String name) {
		String searchFor = '-' + name;

		// Search in all but last args element.
		for (int i = 0; i < args.length - 1; i++) {
			if (args[i].equals(searchFor)) {
				usedParms.add(args[i]);
				return args[i + 1];
			}
		}

		// Not found
		return null;
	}

	/**
	 * If there exists an argument like
	 *     -option
	 * returns 'true'.
	 * Else, returns false.
	 */
	public boolean getOption(String name) {
		String searchFor = '-' + name;

		for (int i = 0; i < args.length; i++) {
			if (args[i].equals(searchFor)) {
				usedParms.add(args[i]);
				return true;
			}
		}

		// Not found
		return false;
	}

	/**
	 * Returns all args which are not associated with
	 * a named parameter.
	 */
	public String[] getUnboundArgs() {
		String[] unbound = new String[args.length];
		int unboundCnt = 0;

		boolean justGotParm = false;
		for (int i = 0; i < args.length; i++) {
			String arg = args[i];
			// if we got a parameter
			if (arg.startsWith("-")) {
				justGotParm = true;
			}
			// if we didn't get a parameter, and last loop we didn't,
			else if (!justGotParm) {
				unbound[unboundCnt++] = arg;
				// if we didn't get a parameter, and last loop we *did*.
			} else {
				justGotParm = false;
			}
		}

		String[] retval = new String[unboundCnt];
		System.arraycopy(unbound, 0, retval, 0, unboundCnt);

		return retval;
	}

	/**
	 * Returns an array containing all of the unused args.
	 */
	public String[] getUnusedArgs() {
		String[] unused = new String[args.length];
		int unusedCnt = 0;

		for (int i = 0; i < args.length; i++) {
			if (args[i].startsWith("-")) {
				if (!usedParms.contains(args[i])) {
					unused[unusedCnt++] = args[i];
				}
			}
		}

		String[] retval = new String[unusedCnt];
		System.arraycopy(unused, 0, retval, 0, unusedCnt);

		return retval;
	}

	private static void p(String n, String v) {
		System.out.println("n=" + n + " v=" + v);
	}

	public static void main(String[] args) {
		CommandLine cl = new CommandLine(args);

		// test with command line
		// like "-verbose -o dir -port 123 a b c"
		//
		p("verbose", "" + cl.getOption("verbose"));
		p("o", "" + cl.getStringArg("o"));
		p("port", "" + cl.getIntArg("port"));

		String[] unboundArgs = cl.getUnboundArgs();
		for (int i = 0; i < unboundArgs.length; i++) {
			p("unbound" + i, unboundArgs[i]);
		}
	}
}

/*
 * $Log: CommandLine.java,v $
 * Revision 1.4  2003/01/06 00:23:49  just
 * moved devenv to linux
 *
 * Revision 1.3  2002/11/14 20:25:19  just
 * reformat of code only
 *
 * Revision 1.2  2001/12/19 22:27:33  just
 * enhancements and test code
 *
 * Revision 1.1  2001/12/19 21:43:46  just
 * new
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/