/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import nl.justobjects.toolkit.io.IO;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.util.Enumeration;
import java.util.Properties;

/**
 * Wraps the XML4J 2 DOM parser.
 * $Id: DOMParser.java,v 1.16 2003/11/19 15:15:15 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
abstract public class DOMParser implements ErrorHandler {
	/** The properties file to be loaded. */
	private static String propertiesPath = "nl/justobjects/toolkit/xml/parser.properties";
	private EntityResolver entityResolver = new EntityResolverImpl();

	/**
	 * Factory method: create instance of a DOM parser
	 */
	public static DOMParser getParser() throws DOMParserException {

		String parserClassName = null;
		Properties parserProperties = null;

		try {
			parserProperties = IO.getPropertiesFromClassPath(propertiesPath);
		} catch (Exception e) {
			p(propertiesPath + " not found in CLASSPATH, trying something else... ");
		}

		if (parserProperties != null) {
			parserClassName = parserProperties.getProperty("domparser");
		}

		if (parserClassName == null) {
			parserClassName = "nl.justobjects.toolkit.xml.JAXPDOMParser";
			p("Parserclassname not found using " + parserClassName);
		}
		p("using "+parserClassName);

		DOMParser domParser = null;
		try {
			domParser = (DOMParser) Class.forName(parserClassName).newInstance();
		} catch (ClassNotFoundException e) {
			throw new DOMParserException("Class " + parserClassName + " cannot be found");
		} catch (InstantiationException e) {
			throw new DOMParserException("Class " + parserClassName + " cannot be instantiated");
		} catch (IllegalAccessException e) {
			throw new DOMParserException("Class " + parserClassName + " cannot be accessed");
		} catch (ClassCastException e) {
			throw new DOMParserException("Class " + parserClassName + " doesn't extend DOMParser");
		}

		// If properties specified, set them.
		if (parserProperties != null) {
			for (Enumeration e = parserProperties.keys(); e.hasMoreElements();) {
				String nextKey = (String) e.nextElement();
				domParser.setProperty(nextKey, parserProperties.getProperty(nextKey));
			}
		}

		// Always use our own EntityResolver; e.g. for filtering cpath:/
		// We could make this a property if really required.
		domParser.setEntityResolver(new EntityResolverImpl());

		return domParser;
	}

	/** Parse the specified file and return a DOM Document. */
	public abstract Document parse(String uri) throws DOMParserException;

	public EntityResolver getEntityResolver() {
		return entityResolver;
	}

	public void setEntityResolver(EntityResolver theEntityResolver) {
		entityResolver = theEntityResolver;
	}

	public abstract void setProperty(String key, String value) throws DOMParserException;

	public void warning(SAXParseException ex) {
		System.err.println("[Warning] " +
				getLocationString(ex) + ": " +
				ex.getMessage());
	}

	/** Error. */
	public void error(SAXParseException ex) {
		System.err.println("[DOMParser Error] " +
				getLocationString(ex) + ": " +
				ex.getMessage());
	}

	/** Fatal error. */
	public void fatalError(SAXParseException ex) throws SAXException {
		System.err.println("[Fatal Error] " +
				getLocationString(ex) + ": " +
				ex.getMessage());
		throw ex;
	}

	/** Returns a string of the location. */
	private String getLocationString(SAXParseException ex) {
		StringBuffer str = new StringBuffer();

		String systemId = ex.getSystemId();
		if (systemId != null) {
			int index = systemId.lastIndexOf('/');
			if (index != -1)
				systemId = systemId.substring(index + 1);
			str.append(systemId);
		}
		str.append(" line:");
		str.append(ex.getLineNumber());
		str.append(" char:");
		str.append(ex.getColumnNumber());

		return str.toString();

	} // getLocationString(SAXParseException):String

	public static void p(String s) {
		// System.out.println("DOMParser-: "+s);
	}

}

/*
 * $Log: DOMParser.java,v $
 * Revision 1.16  2003/11/19 15:15:15  just
 * many changes for using JAXP (with xerces)
 *
 * Revision 1.15  2003/01/06 00:23:49  just
 * moved devenv to linux
 *
 * Revision 1.14  2002/11/14 20:25:20  just
 * reformat of code only
 *
 * Revision 1.13  2001/12/13 23:09:47  just
 * default parser is apache (xerces)
 *
 * Revision 1.12  2001/02/12 23:40:11  just
 * no idea
 *
 * Revision 1.11  2000/12/09 14:38:54  just
 * removed print statements
 *
 * Revision 1.10  2000/11/23 12:53:01  just
 * various changes for entity resolvement
 *
 * Revision 1.9  2000/09/14 11:38:27  just
 * updates
 *
 * Revision 1.8  2000/08/28 00:07:46  just
 * changes for introduction of EntityResolverImpl
 *
 * Revision 1.7  2000/08/24 10:10:43  just
 * added property/feature setting through parser.properties
 *
 * Revision 1.6  2000/08/10 19:26:58  just
 * changes for comments only
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
