/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import org.w3c.dom.*;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

/**
 * XMLPrintVisitor implements the Visitor interface in the visitor design pattern for the
 * purpose of printing in HTML-like format the various DOM-Nodes.
 * <p>In HTML-like printing, only the following Nodes are printed:
 * <DL>
 *  <DT>Document</DT>
 *  <DD>Only the doctype provided on this constructor is written (i.e. no XML declaration, &lt;!DOCTYPE&gt;, or internal DTD).</DD>
 *  <DT>Element</DT>
 *  <DD>All element names are uppercased.</DD>
 *  <DD>Empty elements are written as <code>&lt;BR&gt;</code> instead of <code>&lt;BR/&gt;</code>.</DD>
 *  <DT>Attr</DT>
 *  <DD>All attribute names are lowercased.</DD>
 *  <DT>Text</DT>
 * </DL>
 *
 * <p>The following sample code uses the XMLPrintVisitor on a hierarchy of nodes:
 * <pre>
 *
 * PrintWriter printWriter = new PrintWriter();
 * Visitor htmlPrintVisitor = new XMLPrintVisitor(printWriter);
 * TreeWalker treeWalker = new TreeWalker(htmlPrintVisitor);
 * treeWalker.traverse(document);
 * printWriter.close();
 *
 * </pre>
 *
 * <P>By default, this doesn't print non-specified attributes.</P>
 *
 * @version $Id: XMLPrintVisitor.java,v 1.8 2003/01/06 00:23:49 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 * @see Visitor
 * @see TreeWalker
 */
public class XMLPrintVisitor implements Visitor {
	protected Writer writer = null;

	protected int level = 0;
	protected String doctype = null;


	/**
	 * Constructor for customized encoding and doctype.
	 * @param writer        The character output stream to use.
	 * @param encoding      Java character encoding in use by <VAR>writer</VAR>.
	 * @param doctype       String to be printed at the top of the document.
	 */
	public XMLPrintVisitor(Writer writer, String encoding, String doctype) {
		this.writer = writer;
		this.doctype = doctype;
		// this.isPrintNonSpecifiedAttributes = false;
	}

	/**
	 * Constructor for customized encoding.
	 * @param writer        The character output stream to use.
	 * @param encoding      Java character encoding in use by <VAR>writer</VAR>.
	 */
	public XMLPrintVisitor(Writer writer, String encoding) {
		this(writer, encoding, null);
	}

	/**
	 * Constructor for default encoding.
	 * @param writer  The character output stream to use.
	 */
	public XMLPrintVisitor(Writer writer) {
		this(writer, null, null);
	}

	/**
	 * Constructor for default encoding.
	 * @param writer  The character output stream to use.
	 */
	public XMLPrintVisitor(String fileName) {
		try {
			writer = new FileWriter(fileName);
		} catch (IOException ioe) {
		}
	}

	/**
	 * Writes the <var>doctype</var> from the constructor (if any).
	 * @param   document        Node print as HTML.
	 */
	public void visitDocumentPre(Document document) {
	}

	/**
	 * Flush the writer.
	 * @param   document        Node to print as HTML.
	 */
	public void visitDocumentPost(Document document) {
		write("\n");
		flush();
	}

	/**
	 * Creates a formatted string representation of the start of the specified <var>element</var> Node
	 * and its associated attributes, and directs it to the print writer.
	 * @param   element         Node to print as XML.
	 */
	public void visitElementPre(Element element) {
		this.level++;
		write("<" + element.getTagName());
		NamedNodeMap attributes = element.getAttributes();
		for (int i = 0; i < attributes.getLength(); i++) {
			Attr attr = (Attr) attributes.item(i);
			visitAttributePre(attr);
		}
		write(">\n");
	}

	/**
	 * Creates a formatted string representation of the end of the specified <var>element</var> Node,
	 * and directs it to the print writer.
	 * @param   element         Node to print as XML.
	 */
	public void visitElementPost(Element element) {
		String tagName = element.getTagName();
		// if (element.hasChildNodes()) {
		write("</" + tagName + ">\n");
		// }
		level--;
	}

	/**
	 * Creates a formatted string representation of the specified <var>attribute</var> Node
	 * and its associated attributes, and directs it to the print writer.
	 * <p>Note that TXAttribute Nodes are not parsed into the document object hierarchy by the
	 * XML4J parser; attributes exist as part of a Element Node.
	 * @param   element         Node to print as HTML.
	 */
	public void visitAttributePre(Attr attr) {
		write(" " + attr.getName() + "=\"" + attr.getValue() + "\"");
	}

	/**
	 * Creates a formatted string representation of the specified <var>text</var> Node,
	 * and directs it to the print writer.  CDATASections are respected.
	 * @param     text          Node to print with format.
	 */
	public void visitText(Text text) {
		if (this.level > 0) {
			write(text.getData());
		}
	}

	private void write(String s) {
		try {
			writer.write(s);
		} catch (IOException e) {
		}
	}

	private void flush() {
		try {
			writer.flush();
		} catch (IOException e) {
		}
	}

	/**
	 * Ignore GeneralReference Nodes. Print contents of GeneralReferences.
	 * @param   generalReference        CURRENTLY NOT IMPLEMENTED.

	 public void visitGeneralReferencePre(GeneralReference generalReference) throws Exception {
	 String name = generalReference.getName();
	 if (s_entities.get(name) != null && generalReference.getChildNodes().getLength() == 1) {
	 Node child = generalReference.getFirstChild();
	 if (child.getNodeType() == Node.TEXT_NODE && child.getNodeValue().length() == 1) {
	 write((char)'&' + name + (char)';');
	 throw new ToNextSiblingTraversalException();
	 }
	 }
	 }     */
	static public void main(String[] args) {
		try {
			java.io.PrintWriter printWriter = new java.io.PrintWriter(System.out);
			new TreeWalker(new XMLPrintVisitor(printWriter)).traverse(DOMUtil.parse(args[0]));
		} catch (Exception e) {
			System.out.println("error e=" + e);
		}
	}
}

/* $Log: XMLPrintVisitor.java,v $
/* Revision 1.8  2003/01/06 00:23:49  just
/* moved devenv to linux
/*
/* Revision 1.7  2002/11/14 20:25:20  just
/* reformat of code only
/*
/* Revision 1.6  2001/08/14 11:08:30  kstroke
/* fixes for javadoc comments
/*
/* Revision 1.5  2001/08/14 10:58:03  kstroke
/* fixes for javadoc comments
/*
/* Revision 1.4  2000/12/17 17:02:51  just
/* print newline after element
/*
/* Revision 1.3  2000/08/10 19:26:58  just
/* changes for comments only
/*
 * Revision 1.2  2000/08/07 14:15:09  just
 * no message
 *
 * Revision 1.1.1.1  2000/04/19 20:17:54  just
 * moved to this repository
 *
 * Revision 1.1.1.1  2000/03/16 12:14:15  just
 * no message
 *
 * Revision 1.1.2.1  2000/03/16 11:41:10  just
 * new
 *
 * Revision 1.1  2000/02/17 14:24:16  gerald
 * moved to this server
 *
 * Revision 1.2  1999/10/07 22:07:58  just
 * fixed error in generating close tag
 */

